package com.dky.generate;


import cn.hutool.json.JSONObject;
import com.dky.calculate.*;
import com.dky.modelI.DntdModelI;
import com.dky.tool.ModelTool;
import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.result.MatchedDevice;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

public class BuildHeatingSence implements DntdModelI {

    DecimalFormat decimalFormat = new DecimalFormat("#.00");

    //首先A=由运行/合同容量折合成kW（x0.9）x85%,
    static final Double COEFFICIENT_1 = 0.9;
    static final Double COEFFICIENT_2 = 0.85;

    @Override
    public JSONObject createReport(JSONObject jsonObject2) {
        List<SysDeviceHeatScene> list = ModelTool.specList;
        JSONObject distInfo = (JSONObject) jsonObject2.get("distInfo");
        JSONObject buildInfo = (JSONObject) jsonObject2.get("buildInfo");
        System.out.println("时间: " + Integer.parseInt(buildInfo.get("days").toString()));
        // 运行容量
        Double runCapacity = Double.parseDouble(distInfo.get("runCapacity").toString());
        // 上年最大需量
        Double lastYearNeed = Double.parseDouble(distInfo.get("lastYearNeed").toString());
        // 建筑面积
        Double heatingArea = Double.parseDouble(buildInfo.get("heatingArea").toString());
        // 年采暖时间(天)
        Integer days = Integer.parseInt(buildInfo.get("days").toString());
        // 上年运行费用（万元）
        Double lastYearFee = Double.parseDouble(buildInfo.get("lastYearFee").toString());

        // A=由运行/合同容量折合成kW（x0.9）x85%,
        Double A = runCapacity * COEFFICIENT_1 * COEFFICIENT_2;
        // 冗余容量
        Double B = A - lastYearNeed;
        //投入供热设备总功率C
        AtomicReference<Double> C = new AtomicReference<>();
        //投入供热设备总功率C1
        AtomicReference<Double> C1 = new AtomicReference<>();
        List<Map<String, List<List<MatchedDevice>>>> calScheme = Scheme.calScheme(heatingArea, list);
        List<Map<String, Double>> mapsC = CalC.getC(calScheme);
        for (Map<String, Double> map : mapsC) {
            map.forEach((k, v) -> {
                if (k.contains("电锅炉")) {
                    C.set(v);
                    System.out.println("C" + C.get());
                } else {
                    C1.set(v);
                    System.out.println("C1" + C1.get());
                }
            });
        }
        Double D = lastYearNeed + C.get();
        Double D1 = lastYearNeed + C1.get();

        List<MatchedDevice> matchedDeviceList;
        Map<Double, List<MatchedDevice>> listMap;
        Double costRatio = 0.2;
        Double effRatio = 0.8;
        //获取计算占比
        try{
            Double costRatio1 = Double.parseDouble(jsonObject2.get("costRatio").toString());
            Double effRatio1 = Double.parseDouble(jsonObject2.get("effRatio").toString());
             if(costRatio1 < 1 && costRatio1 + costRatio1 == 1){
                 costRatio = costRatio1;
                 effRatio = effRatio1;
             }
        }catch (Exception e){
            e.printStackTrace();
        }
        if (D1 < A && A < D) {
            // 判断只能用热泵
            List<SysDeviceHeatScene> deviceList = new ArrayList<>();
            list.parallelStream().forEach((sysDeviceHeatScene -> {
                if (!sysDeviceHeatScene.getDevTechType().contains("电锅炉")) {
                    deviceList.add(sysDeviceHeatScene);
                }
            }));
            listMap = SchemeRating.getOptimalList(Scheme.calScheme(heatingArea, deviceList),costRatio,effRatio);
            matchedDeviceList = SchemeRating.getOptimalScheme(listMap);
        } else {
            listMap = SchemeRating.getOptimalList(calScheme,costRatio,effRatio);
            matchedDeviceList = SchemeRating.getOptimalScheme(listMap);
        }

        List<Map<String, Object>> maps = new ArrayList<>();
        listMap.forEach((k,v)->{
            Map<String, Object> map = new HashMap<>();
            map.put("rating", decimalFormat.format(k));
            map.put("plan", v);
            map.put("planName", v.get(0).getDeviceHeatScene().getDevTechType());
            maps.add(map);
        });

        JSONObject jsonObject = new JSONObject();
        Double startCost                = 0.0;
        Double runCost                  = 0.0;
        Double allCost                  = 0.0;
        Double calculateAnnualCarbon    = 0.0;
        Double electric                 = 0.0;


        List<HashMap<String,Object>> deviceList = new ArrayList<>();

        //封装整体情况
        for (MatchedDevice matchedDevice : matchedDeviceList){
            HashMap<String, Object> map = new HashMap<>();
            map.put("devSubType",matchedDevice.getDeviceHeatScene().getDevSubType());
            map.put("devTechType",matchedDevice.getDeviceHeatScene().getDevTechType());
            map.put("devCount",matchedDevice.getCount());
            map.put("devPrice",matchedDevice.getDeviceHeatScene().getDevPrice());
            deviceList.add(map);
            startCost = startCost + (matchedDevice.getCount() * matchedDevice.getDeviceHeatScene().getDevPrice());
            runCost = runCost + (BuildHeatingModel.getRunCost(matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevPower(), days, matchedDevice.getDeviceHeatScene().getDevSubstituteLaborCost()));
            allCost = allCost + (BuildHeatingModel.getYearCost(matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevPrice(), matchedDevice.getDeviceHeatScene().getDevServiceLife(), matchedDevice.getDeviceHeatScene().getDevPower(), days, matchedDevice.getDeviceHeatScene().getDevSubstituteLaborCost()));
            calculateAnnualCarbon = calculateAnnualCarbon + (BuildHeatingModel.calculateAnnualCarbonReduction(lastYearFee, matchedDevice.getDeviceHeatScene().getLaborCost(), matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevPower(), days));
            electric = electric + (BuildHeatingModel.getElectric(matchedDevice.getCount(), matchedDevice.getDeviceHeatScene().getDevPower(), days));
        }
        //初次投资费用
        jsonObject.put("startCost", decimalFormat.format(startCost));
        //年运行费用
        jsonObject.put("yearRunCost", decimalFormat.format(runCost));
        //年总费用
        jsonObject.put("yearCost", decimalFormat.format(allCost));
        //年减碳量
        jsonObject.put("calculate", decimalFormat.format(calculateAnnualCarbon));
        //替代电量
        jsonObject.put("electric", decimalFormat.format(electric));
        //封装需配置设备情况
        jsonObject.put("deviceList",deviceList);

        //封装方案优势
        jsonObject.put("safety", BuildHeatingAdvantage.safety());
        jsonObject.put("economy", BuildHeatingAdvantage.economy(startCost, allCost, runCost, lastYearFee));
        jsonObject.put("intelligence", BuildHeatingAdvantage.intelligence());
        jsonObject.put("environment", BuildHeatingAdvantage.environment(Double.valueOf(decimalFormat.format(calculateAnnualCarbon))));

        //封装方案评分
        jsonObject.put("matchedDeviceList", maps);

        return jsonObject;
    }
}
